#![feature(iter_map_windows)]
use iced::{
    clipboard, widget::{self, column, combo_box, row, text_input}, Element, Task
};
use std::{env, fs, path::PathBuf};
use texts::*;

mod texts;

pub const BIBLE_DIRECTORY: &str = "./Holy-Bible-XML-Format";

fn main() -> iced::Result {
    // CLI
    let arg = env::args().collect::<Vec<String>>();
    if let Some(query) = arg.get(1) {
        if query.starts_with("H") | query.starts_with("h") {
            if let Some(found) = strong::get(query) {
                print!("{found}");
                return Ok(());
            }
        }
        if let Some(second_query) = arg.get(2) {
            // if let Some(f) = bible::get( query, second_query, vec![format!("{BIBLE_DIRECTORY}/EnglishNASBBible.xml").into()],
            // ) {
            //   println!("{}", f);
            // }
            return Ok(())
        }
    }

    // GUI
    iced::run("Bible Search", State::update, State::view)
}

#[derive(Debug, Clone)]
enum Message {
    BibleSearchInput(String),
    BibleSelected(String),
    BibleSearchSubmit,
    CopyText,
    Clear,
    WordSearchInput(String),
    SubmitWordSearch,
}

struct State {
    files: combo_box::State<String>,
    selected_file: Option<String>,
    bible_selected: Option<bible::Bible>,

    bible_search: String,
    body: Option<String>,

    word_search: String,
}
impl Default for State {
    fn default() -> Self {
        let files = if let Ok(contents) = fs::read_dir(BIBLE_DIRECTORY) {
            let paths = contents
                .into_iter()
                .filter_map(|f| f.ok())
                .map(|f| (f.path(), f.file_type()))
                .filter(|(_, f)| f.is_ok())
                .map(|(a, b)| (a, b.unwrap()))
                .filter(|(_, f)| f.is_file())
                .map(|(f, _)| f)
                .filter(|f| f.extension().is_some())
                .filter(|f| f.extension().unwrap() == "xml")
                .collect::<Vec<PathBuf>>();
            paths
        } else {
            todo!()
        };
        let files = files
            .iter()
            .filter_map(|f| f.to_str())
            .map(|f| f.to_owned())
            .collect::<Vec<String>>();

        Self {
            files: combo_box::State::new(files),
            selected_file: None,
            bible_selected: None,

            bible_search: String::new(),
            body: None,

            word_search: String::new(),
        }
    }
}

impl State {
    fn update(&mut self, msg: Message) -> Task<Message> {
        match msg {
            Message::BibleSelected(file) => {
                if let Ok(contents) = fs::read_to_string(&file) {
                    if let Ok(bible) = quick_xml::de::from_str::<bible::Bible>(&contents) {
                        // State is held technically in this variable, although
                        // the Bible is held in memory in it's own variable.
                        self.selected_file = Some(format!("{bible}"));
                        self.bible_selected = Some(bible);
                        // automatically focus the search bar when done here
                        return text_input::focus("bible-search");
                    }
                }
            }
            Message::BibleSearchInput(query) => self.bible_search = query,
            Message::WordSearchInput(query) => self.word_search = query,
            Message::BibleSearchSubmit => {
                let mut splits = self.bible_search.split_whitespace();
                let book = splits.next();
                let location = splits.next();

                if let (Some(book), Some(chap_and_ver), Some(bible)) =
                    (book, location, &self.bible_selected)
                {
                    self.body = bible::get(book, chap_and_ver, vec![bible]);
                }
            }
            Message::SubmitWordSearch => {
                println!("todo: search for {}", self.word_search);
            }
            Message::CopyText => {
                if let Some(text) = &self.body {
                    return clipboard::write::<Message>(parse(text));
                }
            }
            Message::Clear => {
                self.body = None;
                self.bible_search = String::new();
                return text_input::focus("bible-search");
            }
        };
        Task::none()
    }

    fn view(&self) -> Element<Message> {
        let msg = if let Some(body) = self.body.clone() {
            parse(&body)
        } else {
            String::new()
        };

        column![
            widget::combo_box(
                &self.files,
                "Select Bible",
                self.selected_file.as_ref(),
                Message::BibleSelected
            ),
            widget::text_input("Search query, ie: Gen 1:1", &self.bible_search)
                .on_input(Message::BibleSearchInput)
                .on_submit(Message::BibleSearchSubmit)
                .id("bible-search"),
            widget::text_input("Word Search", &self.word_search)
                .on_input(Message::WordSearchInput)
                .on_submit(Message::SubmitWordSearch)
                .id("word-search"),
            row![
                widget::button("Copy").on_press(Message::CopyText),
                widget::button("Clear").on_press(Message::Clear),
            ],
            widget::text(msg),
        ]
        .into()
    }
}

// parse out terminal style codes
fn parse(input: &str) -> String {
    let magic_start_indices = input
        .bytes()
        .into_iter()
        .enumerate()
        .map_windows(|[(ai, a), (_, b)]| {
            // 27 91 is the start sequence for a color/style
            if *a == 27 && *b == 91 {
                Some(*ai)
            } else {
                None
            }
        })
        .filter_map(|f| f)
        .collect::<Vec<usize>>();

    let mut to_remove = Vec::new();
    for start in magic_start_indices {
        let range = &input[start..];
        // "m" is the end of sequence
        if let Some(end) = range.find("m") {
            let end = start + end;
            let code = &input[start..=end];

            match code {
                "\x1B[1m" => {}   //style_bold
                "\x1B[4m" => {}   //style_underline
                "\x1B[0m" => {}   //style_reset
                "\x1B[30m" => {}  //color_black
                "\x1B[31m" => {}  //color_red
                "\x1B[32m" => {}  //color_green
                "\x1B[33m" => {}  //color_yellow
                "\x1B[34m" => {}  //color_blue
                "\x1B[35m" => {}  //color_magenta
                "\x1B[36m" => {}  //color_cyan
                "\x1B[37m" => {}  //color_white
                "\x1B[90m" => {}  //color_bright_black
                "\x1B[91m" => {}  //color_bright_red
                "\x1B[92m" => {}  //color_bright_green
                "\x1B[93m" => {}  //color_bright_yellow
                "\x1B[94m" => {}  //color_bright_blue
                "\x1B[95m" => {}  //color_bright_magenta
                "\x1B[96m" => {}  //color_bright_cyan
                "\x1B[97m" => {}  //color_bright_white
                "\x1B[39m" => {}  //color_reset
                "\x1B[40m" => {}  //bg_black
                "\x1B[41m" => {}  //bg_red
                "\x1B[42m" => {}  //bg_green
                "\x1B[43m" => {}  //bg_yellow
                "\x1B[44m" => {}  //bg_blue
                "\x1B[45m" => {}  //bg_magenta
                "\x1B[46m" => {}  //bg_cyan
                "\x1B[47m" => {}  //bg_white
                "\x1B[100m" => {} //bg_bright_black
                "\x1B[101m" => {} //bg_bright_red
                "\x1B[102m" => {} //bg_bright_green
                "\x1B[103m" => {} //bg_bright_yellow
                "\x1B[104m" => {} //bg_bright_blue
                "\x1B[105m" => {} //bg_bright_magenta
                "\x1B[106m" => {} //bg_bright_cyan
                "\x1B[107m" => {} //bg_bright_white
                "\x1B[49m" => {}  //bg_reset
                _ => {}
            }
            to_remove.push((start, end));
        }
    }
    to_remove.reverse();
    let mut modified = input.to_owned();

    for (s,e) in to_remove {
        let front = modified[..s].to_string();
        let back = modified[e+1..].to_string();
        modified = front + &back;
    }
    
    modified
}

